rawfile

package
v0.0.0-...-1a0588e Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Apr 29, 2020 License: GPL-3.0 Imports: 4 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func BlockingRead

func BlockingRead(fd int, b []byte) (int, error)

BlockingRead reads from a file descriptor that is set up as non-blocking. If no data is available, it will block in a poll() syscall until the file descirptor becomes readable.

func GetMTU

func GetMTU(name string) (int, error)

GetMTU determines the MTU of a network interface device

func NonBlockingWrite

func NonBlockingWrite(fd int, buf []byte) error

NonBlockingWrite writes the given buffer to a file descriptor. It fails if partial data is written

func NonBlockingWrite2

func NonBlockingWrite2(fd int, b1, b2 []byte) error

NonBlockingWrite2 writes up to two byte slices to a file descriptor in a single syscall. It fails if partial data is written

Types

This section is empty.

Jump to