Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func BlockingRead ¶
BlockingRead reads from a file descriptor that is set up as non-blocking. If no data is available, it will block in a poll() syscall until the file descirptor becomes readable.
func NonBlockingWrite ¶
NonBlockingWrite writes the given buffer to a file descriptor. It fails if partial data is written
func NonBlockingWrite2 ¶
NonBlockingWrite2 writes up to two byte slices to a file descriptor in a single syscall. It fails if partial data is written
Types ¶
This section is empty.
Click to show internal directories.
Click to hide internal directories.